<?php

namespace Teknisa\PLA\Modules\Cardapio\Controllers;

use \DateTime;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\Environment;
use Teknisa\GER\Services\SMTPService;
use Zeedhi\Framework\DTO\Response\Error;
use TecFood\Services\User as UserService;
use Teknisa\Libs\Util\SMTP;

use Teknisa\PLA\Util\UtilSMTP;

class Pla07500LiberacaoCardapioFiliais extends \Zeedhi\Framework\Controller\Report{
    
    protected $connection;
    protected $pla07500LiberacaoCardapioService;
    protected $newCode;
    protected $environment;
    protected $smtpService;
    protected $util;
    protected $birtStrategy;
    protected $entityManager;
    protected $authService;

    public function __construct($reportService, $connection, $pla07500LiberacaoCardapioService, $environment, $newCode, $util, $birtStrategy, $entityManager, $authService){
        $this->reportService                    = $reportService;
        $this->connection                       = $connection;
        $this->pla07500LiberacaoCardapioService = $pla07500LiberacaoCardapioService;
        $this->environment                      = $environment;
        $this->newCode                          = $newCode;
        $this->util                             = $util;
        $this->birtStrategy                     = $birtStrategy;
        $this->entityManager                    = $entityManager;
        $this->authService                      = $authService;
        $this->smtpService = new SMTPService(array());
    }
    
    protected $reportMapping = array (
        'PLA07500' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'CDSERVFILI'    => 'P_NMSERVICO',
                        'CDOPERADOR'    => 'P_NMOPERADOR',
                        'DTCARDAPIO'  => 'P_DATACARDAPIO',
                        'NRORG'         => 'P_NRORG',
                    ),
            ),
    );
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        if($row['NMGRUPFILI'] == ""){
            $row['NMGRUPFILI'] = 'T';
        }
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        $row['NRORG'] = $this->environment->getNrOrg();
        
        if($row['ALL_CDFILIAL'] == 'Y'){ // todos
            $filiais = $this->pla07500LiberacaoCardapioService->findFilial($row);
        }else {
            $filiais = $row['NMFILIAL'];
        }
        $row['NMFILIAL'] = $this->util->insertLista($filiais);
        
        if($row['ALL_CDSERVFILI'] == 'Y'){ // todos
            $servicos = $this->pla07500LiberacaoCardapioService->findServico($row);
        }else {
            $servicos = $row['NMSERVICO'];
        }
        $row['CDSERVFILI'] = $this->util->insertLista($servicos);
        
        if($row['ALL_NMOPERADOR'] == 'Y'){ // todos
            $operador = $this->pla07500LiberacaoCardapioService->findOperador($row);
        }else {
            $operador = $row['NMOPERADOR'];
        }
        $row['CDOPERADOR'] = $this->util->insertLista($operador);
    }
    /**
    * Função responsável por retornar os dados de Mes do Cardápio de acordo com
    * os dados enviados do filtro da tela principal.
    * 
    * @param DTO\Request $request Requisição com os dados.
    * @param DTO\Reponse $response Resposta a ser enviada ao requisitante.
    */
    public function findMesCardapio(DTO\Request $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $cdOperador = $this->environment->getCdOperador();
        $cdServicoFilial = array();
        $nrListaFilial = "";
        
        foreach ($conditions as $cond) {
            switch($cond['columnName']){
                case 'CDFILIAL':
                    $cdFilial = $cond['value'];
                    break;
                    
                case 'CDSERVFILI':
                    $cdServfili = $cond['value'];
                    break;
                    
                case 'CDGRUPFILI':
                    $cdGrupfili = $cond['value'];
                    break;
            }
        }
        
        if($cdFilial != 'T'){
            if(!empty($cdFilial))
                $nrListaFilial = $this->util->insertLista($cdFilial);
        }else{
            if($cdGrupfili == 'T'){
                $nrListaFilial = $this->newCode->getCodeByTableName("LISTA", $this->environment->getNrOrg());
                $filiais = $this->pla07500LiberacaoCardapioService->getFiliaisByOper($cdOperador, $nrListaFilial);
            }else{
                $nrListaFilial = $this->newCode->getCodeByTableName("LISTA", $this->environment->getNrOrg());
                $filiais = $this->pla07500LiberacaoCardapioService->getFiliaisByGrupFili($cdOperador, $cdGrupfili, $nrListaFilial);
            }
        }
        
        $cdServfili = is_array($cdServfili) ? implode('_',$cdServfili) : $cdServfili;
        
        $params = $this->pla07500LiberacaoCardapioService->getParamsServFili($nrListaFilial, $cdServfili);
        
        foreach ($params as $key => &$row) {
            $cdServicoFilial[$key] = $row['CDFILIAL'].$row['CDSERVFILI'];
        }
        
        $nrLista = $this->util->insertLista($cdServicoFilial);
        
        $dataSet = $this->pla07500LiberacaoCardapioService->findMesCardapio($nrLista);
        $response->addDataSet(new DataSet("pla07500_mes_cardapio", $dataSet));
    }
    
    private function prepareQueryLikeAll($condition){
        $queryLikeAll = '';
        $propertysConcat = array(
            "CDFILIAL" => false,
            "NMFILIAL" => "CDFILIAL_NMFILIAL"
        );
        $assocFieldToTable = array(
            "CDFILIAL_NMFILIAL" => array(
                'tableName' => 'FL',
                'concat' => ' - '
            ),
            "CDSERVFILI" => array(
                'tableName' => 'SE'
            ),
            "NMSERVICO" => array(
                'tableName' => "SE"
            ),
            "DTPREVLIBECARD" => array(
                "tableName" => "CA",
                "format" => 'TO_CHAR(tableName.NAME_COLUMN, \'DD/MM/YYYY DD/MM/YYYY DY\')'
            ),
            "DTLIBECARDMATR" => array(
                "tableName" => "CTRL",
                "format"    => 'TO_CHAR(tableName.NAME_COLUMN, \'DD/MM/YYYY HH24:MI:SS\')'
            )
        );
        if($condition['operator'] == 'MAPPED_LIKE_ALL'){
            $columns = explode("|", $condition['columnName']);
            $columnsToLike = array();
            foreach($columns as $column){
                $item = '';
                $attr = array();
                $ignore = false;
                $concat = false;
                if(isset($propertysConcat[$column])){
                    if($propertysConcat[$column] !== false){
                        $attr = $assocFieldToTable[$propertysConcat[$column]];
                        $concat = true;
                    } else {
                        $ignore = true;
                    } 
                } else {
                     $attr = $assocFieldToTable[$column];
                }
                if(!$ignore){
                    if(isset($attr['tableName'])){
                        $item = $attr['tableName'].'.';
                        if(isset($attr['format'])){
                            $item = $attr['format'];
                            $item = str_replace('tableName', $attr['tableName'], $item);
                            $item = str_replace('NAME_COLUMN', $column, $item);
                        }
                        if($concat && !$ignore && isset($attr['concat'])){
                            $columnsTable = explode('_',$propertysConcat[$column]);
                            $item .= $columnsTable[0].'||\''.$attr['concat'].'\'||'.$attr['tableName'].'.'.$columnsTable[1]; 
                        } else if(!isset($attr['format'])){
                            $item .= $column; 
                        }
                    }
                    array_push($columnsToLike, $item);
                }
            }
            $columnsToLike = implode('||\'_\'||', $columnsToLike);
            $queryLikeAll = 'AND '. 'UPPER('.$columnsToLike.')' . ' LIKE ' . 'UPPER(\''.$condition['value'][0]['_ALL'].'\')';
        }
        return $queryLikeAll;
    }
    
    
    /**
    * Função responsável por retornar os dados de Mes do Cardápio de acordo com
    * os dados enviados do filtro da tela principal.
    * 
    * @param DTO\Request $request Requisição com os dados.
    * @param DTO\Reponse $response Resposta a ser enviada ao requisitante.
    */
    public function findCardapio(DTO\Request $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $nrLista = "";
        $dtCardapio = '';
        $queryLikeAll = '';
        foreach ($conditions as $cond) {
            switch($cond['columnName']){
                case 'DTMESCARD':
                    $dtCardapio = $cond['value'];
                    break;
                case 'NRLISTA':
                    $nrLista = $cond['value'];
                    break;
            }
            $queryLikeAll = $this->prepareQueryLikeAll($cond);
        }
        if($this->verificaWhereInvalido($queryLikeAll))
            $queryLikeAll = "";
        try{
            $dataSet = array();
            $dataSet = $this->pla07500LiberacaoCardapioService->findCardapio($nrLista, $dtCardapio, $queryLikeAll);
        } catch(\Exception $e){
            throw $e;
        }
        
        $response->addDataSet(new DataSet("pla07500_cardapio", $dataSet));
    }
    
    public function retornaTipoOperador() {
        $row = array(
            'CDOPERADOR'   => $this->environment->getCdOperador(),
            'AUTENTICACAO' => array(
    			'COLUMN_NAME'=> null,
    			'ALIAS_NAME'=> 'LRCD'
		    )
        );
        $permission = $this->authService->returnOperatorInGroup($row);

        return $permission;
    }
    
    /**
    * Função responsável por verificar existe registro de comensal para os dados
    * filtrados na tela principal, e se caso exisitir realizar a inserção na 
    * tabela CARDADIA.
    * 
    * @param DTO\Request $request Requisição com os dados.
    * @param DTO\Reponse $response Resposta a ser enviada ao requisitante.
    */
    public function verifyFilterDataMainGrid(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        
        
        $nrLista = $row['NRLISTA'];
        $dtCardapio = $row['DTMESCARD'];

        //Verifica comensal estimado.
        try{
            $permission = $this->retornaTipoOperador();
            $response->addDataSet(new DataSet ('permission', array('isValid' => $permission)));
            $comensal = array();
            $comensal = $this->pla07500LiberacaoCardapioService->returnComensal($dtCardapio, $nrLista);
            
            
        } catch(\Exception $e){
            throw $e;
        }
        
        if(sizeof($comensal)>0){
            foreach($comensal as $key => &$com){
                $nrcardapio = $this->newCode->getCodeByTableName('CARDAPIO', $this->environment->getNrOrg());
                $com['NRCARDAPIO'] = $nrcardapio;
            }
            
            try{
                $this->pla07500LiberacaoCardapioService->insertCardadia($comensal, $this->environment->getCdOperador(), $dtCardapio);
            } catch (\Exception $e){
                $message = '';
                
                if(strpos($e->getMessage(), 'ORA-20501')){
                    $message = "Cardápio está sendo manipulado por outro usuário.";
                } else if(strpos($e->getMessage(), 'ORA-20500')){
                    $message = "Erro ao inserir cardápio do dia. Registro duplicado.";
                }
                
                $response->setError(new Error($message, 1));
            }
        } 
    }
    
    /**
    * Função responsável por liberar o cardápio, fazendo as verificações e 
    * apropriando cada cardápio à liberação adequada (inserção de novo registro
    * na tabela ou atualização de registro existente).
    * 
    * @param DTO\Request $request Requisição com os dados.
    * @param DTO\Reponse $response Resposta a ser enviada ao requisitante.
    */
    public function liberarCard(DTO\Request $request, DTO\Response $response){
        $requestDataSet = $request->getDataSet()->getRows();
        $dtCardapio = $requestDataSet[0]['DATA'];
        $cardapios = $requestDataSet[1]['ROWS'];
        
        //Divide o array de cardapios em grupos de 999 registros.
        //Evitando erro IN clause Oracle DB v<=10g.
        $cardapiosGroup = array_chunk($cardapios, 999);
        $cardapioReleaseDates = array();
        
        
        foreach ($cardapiosGroup as $currentCardapioGroup) {
            $resultCard = $this->pla07500LiberacaoCardapioService->getCardapioReleaseDates($currentCardapioGroup, $dtCardapio);
            $cardapioReleaseDates = array_merge($cardapioReleaseDates, $resultCard);
        }
        
        
        $releasedCardElements = array_map(function($rCard){
            return array(
                "CDFILIAL" => $rCard['CDFILIAL'], 
                "CDSERVFILI" => $rCard['CDSERVFILI']
            );
        }, $cardapioReleaseDates);
        
        
        
        //Filtra registros que serão incluidos.
        $cardToInclude = array_filter($cardapios, function($card) use ($releasedCardElements){
            $cardElement = array(
                "CDFILIAL" => $card['CDFILIAL'], 
                "CDSERVFILI" => $card['CDSERVFILI']
            );
            
            return !in_array($cardElement, $releasedCardElements);
        });
        
        $cardToInclude = array_map(function($elem){
            return array(
                "CDFILIAL" => $elem['CDFILIAL'], 
                "CDSERVFILI" => $elem['CDSERVFILI']
            );
        }, $cardToInclude);
        
        
        //Filtra registros existentes que serão alterados.
        $cardToUpdate = array();
        if(sizeof($cardapioReleaseDates)>0){
            $cardToUpdate = array_filter($cardapios, function($card, $index) use ($releasedCardElements){
                $cardElement = array(
                    "CDFILIAL" => $card['CDFILIAL'], 
                    "CDSERVFILI" => $card['CDSERVFILI']
                );
                
                return (in_array($cardElement, $releasedCardElements) &&
                            !isset($releasedCardElements[$index]['DTLIBECARDMATR']));
            }, ARRAY_FILTER_USE_BOTH);
            
            $cardToUpdate = array_map(function($elem){
                return array(
                    "CDFILIAL" => $elem['CDFILIAL'], 
                    "CDSERVFILI" => $elem['CDSERVFILI']
                );
            }, $cardToUpdate);
        }
        
        
        foreach ($cardToInclude as $card) {
            $this->pla07500LiberacaoCardapioService->releaseCardInclude($card, $dtCardapio);
        }
        
        foreach ($cardToUpdate as $card) {
            $this->pla07500LiberacaoCardapioService->releaseCardUpdate($card, $dtCardapio);
        }
    }
    
    
    /**
    * Função responsável por cancelar a liberação o cardápio, fazendo as 
    * verificações necessárias.
    * 
    * @param DTO\Request $request Requisição com os dados.
    * @param DTO\Reponse $response Resposta a ser enviada ao requisitante.
    */
    public function cancelLiberarCard(DTO\Request $request, DTO\Response $response){
        $requestDataSet = $request->getDataSet()->getRows();
        $dtCardapio = $requestDataSet[0]['DATA'];
        $cardapios = $requestDataSet[1]['ROWS'];
        
        //Divide o array de cardapios em grupos de 999 registros.
        //Evitando erro IN clause Oracle DB v<=10g.
        $cardapiosGroup = array_chunk($cardapios, 999);
        $cardapioReleaseDates = array();
        
        
        foreach ($cardapiosGroup as $currentCardapioGroup) {
            $resultCard = $this->pla07500LiberacaoCardapioService->getCardapioReleaseDates($currentCardapioGroup, $dtCardapio);
            $cardapioReleaseDates = array_merge($cardapioReleaseDates, $resultCard);
        }
        
        
        $releasedCardElements = array_map(function($rCard){
            return array(
                "CDFILIAL" => $rCard['CDFILIAL'], 
                "CDSERVFILI" => $rCard['CDSERVFILI']
            );
        }, $cardapioReleaseDates);
        
        
        //Filtra registros existentes que serão alterados.
        $cardsToCancel = array();
        if(sizeof($cardapioReleaseDates)>0){
            
            
            $cardsToCancel = array_filter($cardapios, function($card, $index) use ($releasedCardElements, $cardapioReleaseDates){
                $cardElement = array(
                    "CDFILIAL" => $card['CDFILIAL'], 
                    "CDSERVFILI" => $card['CDSERVFILI']
                );
                
                return (in_array($cardElement, $releasedCardElements) &&
                            isset($cardapioReleaseDates[$index]['DTLIBECARDMATR']) &&
                            !isset($cardapioReleaseDates[$index]['DTAPROCARDFILI']));
                
                
                
            }, ARRAY_FILTER_USE_BOTH);
            
            
            $cardsToCancel = array_map(function($elem){
                return array(
                    "CDFILIAL" => $elem['CDFILIAL'], 
                    "CDSERVFILI" => $elem['CDSERVFILI']
                );
            }, $cardsToCancel);
            
        }
        
        foreach ($cardsToCancel as $card) {
            $this->pla07500LiberacaoCardapioService->cancelLiberarCard($card, $dtCardapio);
        }
    }
    
    /**
    * Função responsável por enviar email automaticamente após liberação ou 
    * cancelamento de cardápios.
    * 
    * @param DTO\Request $request Requisição com os dados.
    * @param DTO\Reponse $response Resposta a ser enviada ao requisitante.
    */
    public function sendEmail(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();

        $smtp = new UtilSMTP($this->connection, $this->environment);
        $smtpParams = $smtp->findSmtpParamsUnit($this->environment->getCdFilial());
 
        $this->smtpService = new SMTPService($smtpParams);

        $this->smtpService->setFrom($smtpParams['from']);
        //$this->smtpService->setFromName($this->environment->getCurrentUserName());
        //$this->smtpService->setNmAddress($row->get('mailRecipient')[0]);
        $this->smtpService->setSubject($row->get('mailSubject'));
        $this->smtpService->sendMail($row->get('mailBody'), $row->get('DSEMAILOPER'));
    }
    
    private function verificaWhereInvalido($where){
        $pattern ='/\b(SELECT|INSERT|UPDATE|DELETE|MERGE|CREATE|GRANT|DROP).{1,}(FROM|SET|TABLE|TRIGGER|DATABASE|PROCEDURE|FUNCTION|VIEW|JOB)\b/i';
        return preg_match_all($pattern, $where, $matches);
    }
}